#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu





import threading
import multiprocessing
from functools import wraps
from Queue import Empty

__all__ = ['process', 'thread']


def process(*proxy_args, **proxy_kwargs):

    fire = proxy_kwargs.pop('fire', False) or proxy_args or proxy_kwargs

    def decorator(func):

        if fire:

            @wraps(func)
            def fun(*args, **kwargs):
                func(*args, **kwargs)

            fun.process = multiprocessing.Process(target=fun, args=proxy_args,
                                                   kwargs=proxy_kwargs)
            fun.process.start()

            return fun

        else:

            @wraps(func)
            def wrapper(*args, **kwargs):
                process = multiprocessing.Process(target=func, args=args,
                                                  kwargs=kwargs)
                process.start()
                return process

            return wrapper

    return decorator


def thread(*proxy_args, **proxy_kwargs):

    fire = proxy_kwargs.pop('fire', False) or proxy_args or proxy_kwargs

    def decorator(func):

        if fire:

            @wraps(func)
            def fun(*args, **kwargs):
                func(*args, **kwargs)

            fun.thread = threading.Thread(target=fun, args=proxy_args,
                                           kwargs=proxy_kwargs)
            fun.thread.start()

            return fun

        else:

            @wraps(func)
            def wrapper(*args, **kwargs):
                thread = threading.Thread(target=func, args=args,
                                          kwargs=kwargs)
                thread.start()
                return thread

            return wrapper

    return decorator


class StopWorker():
    pass


def worker(block=True, timeout=0.1, method='process'):
    """
        Turn a function into a worker:

            from parallel import worker

            @worker()
            def test(mot):
                print "in %s" % mot
                return mot

            process = test.start()

            for x in range(10):
                process.put(x)

            for x in range(10):
                print "out %s" % process.get()

            process.stop()

        Which outputs:

            in 0
            out 0
            in 1
            in 2
            out 1
            in 3
            out 2
            in 4
            out 3
            out 4
            in 5
            out 5
            in 6
            out 6
            in 7
            out 7
            in 8
            out 8
            in 9
            out 9

        If an exception occures, it will be send back in the message queue.

        So you may want to check the result with isintance().
    """

    def decorator(func):

        if method == 'thread':
            Queue = threading.Queue
            Manager = threading.Thread
            Error = threading.TimeoutError
        else:
            Queue = multiprocessing.Queue
            Manager = multiprocessing.Process
            Error = multiprocessing.TimeoutError

            in_queue = Queue()
            out_queue = Queue()

            def main_loop():

                while True:
                    try:

                        res = in_queue.get(block, timeout)

                        if isinstance(res, StopWorker):
                            break

                        out_queue.put(func(res))

                    except (Error, Empty):
                        pass
                    except (StopWorker, KeyboardInterrupt):
                        break
                    except Exception as e:
                        out_queue.put(e)

            @wraps(func)
            def wrapper(*args, **kwargs):
                return func(*args, **kwargs)

            def get(block=block, timeout=timeout):
                res = out_queue.get(block, timeout)
                if isinstance(res, Exception):
                    raise res
                return res

            wrapper.manager = Manager(target=main_loop)
            wrapper.manager.get = get
            wrapper.manager.stop = lambda: in_queue.put(StopWorker(), block, timeout)
            wrapper.manager.put = lambda x: in_queue.put(x, block, timeout)
            wrapper.TimeoutError = Error
            wrapper.start = lambda: wrapper.manager.start() or wrapper.manager

            return wrapper

    return decorator



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu


__version__ = "0.5"

from strings import (slugify, normalize, escape_html,
                     unescape_html, json_dumps, json_loads)
from structs import (chunks, get, dmerge, sset, dswap, window,
                     subdict, first, first_true)
from objects import attr, import_from_path, Null
from utils import to_timestamp
from hack import decorator_with_args



# -*- coding: utf-8 -*-
#!/usr/bin/env python
# vim: ai ts=4 sts=4 et sw=4 nu

u"""

    Various tools to manipulate strints. Features tools to normalize and
    slugify strings. Also some ways to escape HTML and a JSON serializer
    that deals with datetime objects.

    Here is an example of the generic slugifier utility (as for 'normalize',
    'escape_html', 'unescape_html' and "json_dumps" are pretty straightforward
    and don't need that much of an explanation. Ok, it's just because I'm
    too lazy to write it down really).

    Works out of the box for Latin-based scripts.

    Example:

        >>> from strings import slugify
        >>> slugify(u"C'est Noël !")
        u'cest-noel'
        >>> slugify(u"C'est Noël !", separator="_")
        u'cest_noel'

    It will handle all unicode equivalences if (and only if) the optional
    unidecode library is installed.

    Example:

        >>> slugify(u"北亰")
        u'bei-jing'

    More about it:
      - http://en.wikipedia.org/wiki/Unicode_equivalence;
      - http://pypi.python.org/pypi/Unidecode.

    If you do have unidecode installed, but wish not to use it, use the
    unicodedata_slugify fonction:

        >>> slugify(u"Héllø Wörld") # slugify() uses unidecode if it can
        u'hello-world'
        >>> unicodedata_slugify(u"Héllø Wörld") # this will more limited
        u'hell-world'

    In case you wish to keep the non ASCII characters "as-is", use
    unicode_slugify():

    >>> print unicode_slugify(u"C'est Noël !")
    cest-noël

"""

import re
import json
import unicodedata

from datetime import datetime, timedelta, date, time
from xml.sax.saxutils import escape, unescape

from utils import CLASSIC_DATETIME_FORMAT, CLASSIC_DATETIME_PATTERN



def unicode_slugify(string, separator=r'-'):
    ur"""
    Slugify a unicode string using to normalize the string, but without trying
    to convert or strip non ASCII characters.

    Example:

        >>> print unicode_slugify(u"Héllø Wörld")
        héllø-wörld
        >>> unidecode_slugify(u"Bonjour, tout l'monde !", separator="_")
        u'bonjour_tout_lmonde'
        >>> unidecode_slugify(u"\tStuff with -- dashes and...   spaces   \n")
        u'stuff-with-dashes-and-spaces'
    """

    string = re.sub(r'[^\w\s' + separator + ']', '', string, flags=re.U)
    string = string.strip().lower()
    return unicode(re.sub(r'[' + separator + '\s]+',
                   separator, string, flags=re.U))


def unicodedata_slugify(string, separator=r'-'):
    ur"""
    Slugify a unicode string using unicodedata to normalize the string.

    Example:

        >>> unicodedata_slugify(u"Héllø Wörld")
        u'hell-world'
        >>> unidecode_slugify(u"Bonjour, tout l'monde !", separator="_")
        u'bonjour_tout_lmonde'
        >>> unidecode_slugify(u"\tStuff with -- dashes and...   spaces   \n")
        u'stuff-with-dashes-and-spaces'
    """

    string = unicodedata.normalize('NFKD', string).encode('ascii', 'ignore')
    string = re.sub(r'[^\w\s' + separator + ']', '', string).strip().lower()
    return unicode(re.sub(r'[' + separator + '\s]+', separator, string))


def unidecode_slugify(string, separator=r'-'):
    ur"""
    Slugify a unicode string using unidecode to normalize the string.

    Example:

        >>> unidecode_slugify(u"Héllø Wörld")
        u'hello-world'
        >>> unidecode_slugify(u"Bonjour, tout l'monde !", separator="_")
        u'bonjour_tout_lmonde'
        >>> unidecode_slugify(u"\tStuff with -- dashes and...   spaces   \n")
        u'stuff-with-dashes-and-spaces'
    """

    string = unidecode.unidecode(string)
    string = re.sub(r'[^\w\s' + separator + ']', '', string).strip().lower()
    return unicode(re.sub(r'[' + separator + '\s]+', separator, string))


def unicodedata_normalize(string):
    return unicodedata.normalize('NFKD', string).encode('ascii', 'ignore')


def unidecode_normalize(string):
    return unidecode.unidecode(string)


try:
    import unidecode
    slugify = unidecode_slugify
    normalize = unidecode_normalize
except ImportError:
    slugify = unicodedata_slugify
    normalize = unicodedata_normalize


def escape_html(text, additional_escape={'"': "&quot;", "'": "&apos;"}):
    """
        Turn HTML tag caracters into HTML entities.


        Example:

            >>> escape_html("<strong>Ben & Jelly's !</strong>")
            '&lt;strong&gt;Ben &amp; Jelly&apos;s !&lt;/strong&gt;'

    """
    return escape(text, additional_escape)


def unescape_html(text, additional_escape={"&quot;": '"', "&apos;": "'"}):
    """
        Turn HTML tag entities into ASCII caracters.

        Example:

            >>> unescape_html('&lt;strong&gt;Ben &amp; Jelly&apos;s !&lt;/strong&gt;')
            "<strong>Ben & Jelly's !</strong>"
    """
    return unescape(text, additional_escape)



class JSONEncoder(json.JSONEncoder):
    """
        Json encoder with date and time handling.

        You should use naive datetime only. If you have timezone information,
        store them in a separate field.
    """


    DATETIME_FORMAT = CLASSIC_DATETIME_FORMAT
    DATE_FORMAT, TIME_FORMAT = DATETIME_FORMAT.split()
    TIMEDELTA_FORMAT = "timedelta(seconds='%s')"


    def __init__(self, datetime_format=None, date_format=None, time_format=None,
                timedelta_format=None, *args, **kwargs):

        self.datetime_format = datetime_format or self.DATETIME_FORMAT
        self.date_format = date_format or self.DATE_FORMAT
        self.time_format = time_format or self.TIME_FORMAT
        self.timedelta_format = timedelta_format or self.TIMEDELTA_FORMAT

        super(JSONEncoder, self).__init__(self, *args, **kwargs)


    def default(self, obj):

        if isinstance(obj, datetime):
            return obj.strftime(self.datetime_format)

        if isinstance(obj, date):
            return obj.strftime(self.date_format)

        if isinstance(obj, time):
            return obj.strftime(self.time_format)

        if isinstance(obj, timedelta):
            return self.timedelta_format % obj.total_seconds()

        return json.JSONEncoder.default(self, obj)



class JSONDecoder(json.JSONDecoder):
    """
        Json decoder that decode JSON encoded with JSONEncoder
    """

    DATETIME_PATTERN = CLASSIC_DATETIME_PATTERN
    DATE_PATTERN, TIME_PATTERN = DATETIME_PATTERN.split()
    TIMEDELTA_PATTERN = r"timedelta\(seconds='(?P<seconds>\d+(?:\.\d+)*)'\)"


    def __init__(self, datetime_pattern=None, date_pattern=None,
                time_pattern=None, timedelta_pattern=None, datetime_format=None,
                date_format=None, time_format=None, *args, **kwargs):

        self.datetime_format = datetime_format or JSONEncoder.DATETIME_FORMAT
        self.date_format = date_format or JSONEncoder.DATE_FORMAT
        self.time_format = time_format or JSONEncoder.TIME_FORMAT

        self.datetime_pattern = re.compile(datetime_pattern or self.DATETIME_PATTERN)
        self.date_pattern = re.compile(date_pattern or self.DATE_PATTERN)
        self.time_pattern = re.compile(time_pattern or self.TIME_PATTERN)
        self.timedelta_pattern = re.compile(timedelta_pattern or self.TIMEDELTA_PATTERN)

        super(JSONDecoder, self).__init__(object_pairs_hook=self.object_pairs_hook,
                                          *args, **kwargs)

    def object_pairs_hook(self, obj):
        return dict((k, self.decode_on_match(v)) for k, v in obj)


    def decode_on_match(self, obj):
        """
            Try to match the string, and if it fits any date format,
            parse it and returns a Python object.
        """

        string = unicode(obj)

        match = re.search(self.datetime_pattern, string)
        if match:
            return datetime.strptime(match.string, self.datetime_format)

        match = re.search(self.date_pattern, string)
        if match:
            return datetime.strptime(match.string, self.date_format).date()

        match = re.search(self.time_pattern, string)
        if match:
            return datetime.strptime(match.string, self.time_format).time()

        match = re.search(self.timedelta_pattern, string)
        if match:
            return timedelta(seconds=float(match.groupdict()['seconds']))

        return obj


def json_dumps(data, datetime_format=None, date_format=None, time_format=None,
                timedelta_format=None, *args, **kwargs):
    r"""
        Same as Python's json.dumps but also serialize datetime, date, time
        and timedelta.

        Example:
            >>> import datetime
            >>> json_dumps({'test': datetime.datetime(2000, 1, 1, 1, 1, 1)})
            '{"test": "2000-01-01 01:01:01.000000"}'
            >>> json_dumps({'test': datetime.date(2000, 1, 1)})
            '{"test": "2000-01-01"}'
            >>> json_dumps({'test': datetime.time(1, 1, 1)})
            '{"test": "01:01:01.000000"}'
            >>> json_dumps({'test': datetime.timedelta(1, 1)})
            '{"test": "timedelta(seconds=\'86401.0\')"}'
            >>> json_dumps({u'test': datetime.timedelta(1, 1), u'a': [1, 2]})
            '{"test": "timedelta(seconds=\'86401.0\')", "a": [1, 2]}'

    """
    return JSONEncoder(datetime_format, date_format, time_format,
                       timedelta_format, *args, **kwargs).encode(data)


def json_loads(string, datetime_pattern=None, date_pattern=None,
                time_pattern=None, timedelta_pattern=None, datetime_format=None,
                date_format=None, time_format=None, *args, **kwargs):
    r"""
        Same as Python's json.loads, but handles formats from batbelt.json_dumps
        which are currently mainly date formats.

        Example:

            >>> json_loads('{"test": "2000-01-01 01:01:01.000000"}')
            {u'test': datetime.datetime(2000, 1, 1, 1, 1, 1)}
            >>> json_loads('{"test": "2000-01-01"}')
            {u'test': datetime.date(2000, 1, 1)}
            >>> json_loads('{"test": "01:01:01.000000"}')
            {u'test': datetime.time(1, 1, 1)}
            >>> json_loads('{"test": "timedelta(seconds=\'86401.0\')"}')
            {u'test': datetime.timedelta(1, 1)}
            >>> json_loads('{"test": "timedelta(seconds=\'86401.0\')", "a": [1, 2]}')
            {u'test': datetime.timedelta(1, 1), u'a': [1, 2]}

    """
    return JSONDecoder(datetime_pattern, date_pattern, time_pattern,
                       timedelta_pattern, datetime_format, date_format,
                       time_format, *args, **kwargs).decode(string)


def template(tpl, context):
    """
      Use the given a template file, call .format() on it's content,
      and returns it as a string.

      Template file can be a path or a file like object.
    """

    try:
        tpl = open(tpl)
    except TypeError:
        pass

    return tpl.read().format(**context)


def render(tpl, context, target):
    """
      Render the template and write the result in a file.

      Template and target files can be a path or a file like objects.
    """

    try:
        target = open(target, 'w')
    except TypeError:
        pass

    res = template(tpl, context)
    target.write(res)

    target.close()


if __name__ == "__main__":
    import doctest
    doctest.testmod()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu

"""
    The infamous utils.py module filled with functions you don't where else
    to put.
"""


import sys

from datetime import datetime


CLASSIC_DATETIME_FORMAT = '%Y-%m-%d %H:%M:%S.%f'
CLASSIC_DATETIME_PATTERN = r'\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}\.\d{6}'




def to_timestamp(dt):
    """
        Return a timestamp for the given datetime object.

        Example:

            >>> import datetime
            >>> to_timestamp(datetime.datetime(2000, 1, 1, 1, 1, 1, 1))
            946688461
    """
    return (dt - datetime(1970, 1, 1)).total_seconds()


class ImportableItems(list):

    def __init__(self, *args, **kwargs):
        super(ImportableItems, self).__init__(*args, **kwargs)
        self.non_importable_items = {}

    def append(self, item_name):
        self.non_importable_items.pop(item_name, None)
        super(ImportableItems, self).append(item_name)


def import_list(*args):
    """
        Allow to create easily a __all__ listing for a module.

        Returns a value for __all__ and a decorator to add anything
        to it easily.
    """

    importable_items = ImportableItems()
    importable_items.non_importable_items.update(sys._getframe(1).f_globals)
    for item in args:
        importable_items.append(item)


    def importable(func, name=None):

        if name is None:
            try:
                name = func.__name__
            except AttributeError:
                raise ValueError('You must provide a name for '
                                 'this item: %s' % repr(func))
        importable_items.append(name)

        return func

    return importable_items, importable





from setuptools import setup, find_packages

open('MANIFEST.in', 'w').write('\n'.join((
    "include *.rst",
)))

from batbelt import __version__

setup(

    name="batbelt",
    version=__version__,
    packages=find_packages('.'),
    author="Sam et Max",
    author_email="lesametlemax@gmail.com",
    description="A collection of gagdets that makes Python even more powerful.",
    long_description=open('README.rst').read(),
    include_package_data=True,
    classifiers=[
        'Programming Language :: Python',
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: zlib/libpng License",
        "Natural Language :: English",
        "Programming Language :: Python :: 2.7"
    ],
    url="https://github.com/sametmax/Bat-belt"
)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu


import sys

from functools import wraps

from io import BytesIO
from contextlib import contextmanager


@contextmanager
def capture_ouput(stdout_to=None, stderr_to=None):
    """
        Context manager that captures any printed ouput in the 'with' block.

        :Example:

        >>> with capture_ouput() as (stdout, stderr):
        ...    print "hello",
        ...
        >>> print stdout.getvalue().upper()
        HELLO
        >>> with capture_ouput() as (stdout, stderr):  # doctest: +IGNORE_EXCEPTION_DETAIL
        ...    assert False
        ...
        Traceback (most recent call last):
        AssertionError
        >>> from tempfile import NamedTemporaryFile
        >>> f = NamedTemporaryFile(mode="rw+b")
        >>> with capture_ouput(f) as (stdout, stderr):
        ...    print "hello",
        ...
        >>> print stdout.read()
        hello


        .. :warning: this is NOT thread safe.

        .. :note: The file like objects containing the capture are not closed
                  automatically by this context manager. You are responsible
                  to do it.

    It does not capture exception, so they bubble out and print the stack
    trace anyway.
    """

    try:

        stdout, stderr = sys.stdout, sys.stderr
        sys.stdout = c1 = stdout_to or BytesIO()
        sys.stderr = c2 = stderr_to or BytesIO()
        yield c1, c2

    finally:

        sys.stdout = stdout
        sys.stderr = stderr

        try:
            c1.flush()
            c1.seek(0)
        except (ValueError, TypeError):
            pass

        try:
            c2.flush()
            c2.seek(0)
        except (ValueError, TypeError):
            pass


def decorator_with_args(wrap=True,
                        function_assigned=('__module__', '__name__', '__doc__'),
                        function_updated=('__dict__',),
                        decorator_assigned=('__module__', '__name__', '__doc__'),
                        decorator_updated=('__dict__',),
                        ):
    """
        Use this decorator on a wannabe decorator.

        It will turn it into a decorator that accept any arguments and
        wraps the resulting decorated function unless you set wrap=False.

        Usage:

            # You use @decorator_with_args on a function you wish to
            # be a decorator accepting arguments
            @decorator_with_args()
            def your_decorator(func, *args, **kwargs):
                def wrapper():
                    # do stuff
                    return func()
                return wrapper

        Your decorator must accept the function as the first argument, and
        expact the other arguments after that. It doesn't have to be *args,
        **kwargs, it can be any signature, as long as the first argument,
        is the function to decorate.

        One your wannabe decorator decorated, you can use it this way:

            # When you use YOUR decorator, you will be able to pass arguments
            @your_decorator(arg1, arg2, arg3='foo')
            def a_function():
                # do stuff

            # If you don't use arguments, you still need the parenthesis
            @your_decorator()
            def another_function():
                # do stuff

        By default, @decorator_with_args will attempt to apply functools.wraps on
        the wrapper your wannabe decorator returns. If you don't wish that,
        pass wrap=False:

            @decorator_with_args(wrap=False)
            def your_decorator(func, *args, **kwargs):
                def wrapper():
                    # do stuff
                    return func()
                # This will be passed to functools.wraps() if you don't
                # set wrap=False
                return wrapper

        You can also pass the same arguments you would pass to functools.wraps
        directly to @decorator_with_args. They will be passed to along:

            @decorator_with_args(function_assigned=('__module__', '__name__', '__doc__'))
            def your_decorator(func, *args, **kwargs):
                def wrapper():
                    # do stuff
                    return func()
                # this will apply functools.wrap() with assigned being set to
                # ('__module__', '__name__')
                return wrapper

        The params are named function_assigned and function_updated instead of
        just assigned and updated like in functools.wraps.

        Also, @decorator_with_args will ALWAYS apply functools.wraps to the
        wrapper around your wannabe decorator. You can also control what's
        copied by passing decorator_assigned and decorator_updated the same way:

            # functools.wrap will always be applied to your_decorator()
            # but you can choose with which arguments
            @decorator_with_args(decorator_updated=('__dict__',))
            def your_decorator(func, *args, **kwargs):
                def wrapper():
                    # do stuff
                    return func()
                return wrapper

    """
    # decorator() will return this function, wich will be the real decorator
    # called on the wannabe decorator.
    def _decorator(wannabe_decorator):

        # This is the function that will return your wrapped wannabe decorator.
        # Il will add a wrapper that will call your wannabe decorator with
        # the arguments stored in a closure under the hood.
        def decorator_maker(*args, **kwargs):

            # This is the the wrapper around your wannabe decorator. It
            # replaces your function so it can pass arguments to it.
            # We apply @wraps on it so it takes all metadata from
            # the wannabe decorator and attach them to itself.
            @wraps(wannabe_decorator, decorator_assigned, decorator_updated)
            def decorator_wrapper(func):

                # The wrapper calls your wannabe decorator, passing the
                # function to decorate and arguments to it.
                # It will get the wrapper your wannabe decorator returns,
                # and if, wrap=True (default), will apply @wraps on it too.
                d = wannabe_decorator(func, *args, **kwargs)
                if wraps:
                    d = wraps(func, function_assigned, function_updated)(d)
                return d

            return decorator_wrapper

        return decorator_maker

    return _decorator


class MultiStopIteration(StopIteration):
    def throw(self):
        raise self



@contextmanager
def multibreak():
    '''

        Context manager which allow to break multiple nested for loops at once.

        Example:

            >>> with multibreak() as stop:
            ...     for x in range(1, 4):
            ...         for z in range(1, 4):
            ...             for w in range(1, 4):
            ...                 print w
            ...                 if x * z * w == 2 * 2 * 2:
            ...                     print 'stop'
            ...                     stop()
            ...
            1
            2
            3
            1
            2
            3
            1
            2
            3
            1
            2
            3
            1
            2
            stop
    '''

    try:
        yield MultiStopIteration().throw
    except MultiStopIteration:
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu


from collections import MutableSet, deque

from itertools import islice, chain


__all__ = ['chunks', 'dmerge', 'get', 'window', 'dswap', 'subdict', 'first',
           'first_true', 'sset']


def chunks(seq, chunksize, process=iter):
    """
        Yields items from an iterator in iterable chunks.
    """
    it = iter(seq)
    while True:
        yield process(chain([it.next()], islice(it, chunksize - 1)))



def window(iterable, size=2):
    """
        Yields iterms by bunch of a given size, but rolling only one item
        in and out at a time when iterating.
    """
    iterable = iter(iterable)
    d = deque(islice(iterable, size), size)
    yield d
    for x in iterable:
        d.append(x)
        yield d


def dmerge(d1, d2):
    """
        Create a new dictionary being the merge of the two passed as a
        parameter.

        The keys in the second one erases the keys in the first one.
    """
    d = {}
    d.update(d1)
    d.update(d2)
    return d


def dswap(dct):
    """
        Swap key and values of a given dictionary. Return a new dictionary.

        If you have duplicate values, the last one in the dictionary order
        will be used. Since dictionary order is not predictable, you should
        make sure to either remove duplicates values before processing, or
        just make sure loosing some keys is not a problem for you.


        example:

            >>> sorted(dswap({'a': 1, 'b': 2}).items())
            [(1, 'a'), (2, 'b')]
    """
    return dict((value, key) for key, value in dct.iteritems())


def get(data, *keys, **kwargs):
    """
        Extract a data from nested mapping and sequences using a list of keys
        and indices to apply successively. If a key error or an index error
        is raised, returns the default value.

        res = get(data, 'test', 0, 'bla', default="yeah")

        is the equivalent of

        try:
            res = data['test'][0]['bla']
        except (KeyError, IndexError):
            res = "yeah"

    """
    try:
        value = data[keys[0]]

        for key in keys[1:]:
            value = value[key]
    except (KeyError, IndexError, TypeError):
        return kwargs.get('default', None)

    return value


def rename(dct, old_name, new_name):
    """
        Rename a key in a dictionary. No effect if the key does not exists.

        Return the dictiony passed as parameter.
    """

    try:
        dct[new_name] = dct[old_name]
        del dct[old_name]
    except KeyError:
        pass

    return dct


def unpack(dct, *args, **kwargs):
    """
        Return an generator with the values for the given keys or
        a default value.

        :Example:

        >>> a, b, c = unpack(dct, 'a', 'b', 'c', default=1)
        >>> a
        2
        >>> b
        4
        >>> c
        1

    """

    default = kwargs.get('default', None)

    for key in args:
        yield dct.get(key, default)



def subdict(dct, include=(), exclude=()):
    """
        Return a dictionary that is a copy of the given one.

        All tvalues in `include` are used as key to be copied to
         the resulting dictionary.

        You can also pass a list of key to exclude instead by setting
        `exclude`. But you can't use both `include` and `exclude`: if you do,
        `exclude will be ignored`

        Example:

        >>> subdict({1:None, 2: False, 3: True}, [1, 2])
        {1: None, 2: False}
        >>> subdict({1:None, 2: False, 3: True}, exclude=[1, 2])
        {3: True}

    """

    if include:
        return dict((k, v) for k, v in dct.iteritems() if k in include)

    return dict((k, v) for k, v in dct.iteritems() if k not in exclude)


def first(iterable, default=None):
    """
        Return the first item of any iterable. If the iterable is empty,
        return the default value.
    """
    for x in iterable:
        return x
    return default


def first_true(iterable, key=lambda x: x, default=None):
    """
        Return the first item of any iterable for which the key is True.

        By default the key is the entire element.

        If the iterable is empty, return the default value.
    """
    for x in iterable:
        if key(x):
            return x
    return default


KEY, PREV, NEXT = range(3)


class sset(MutableSet):
    """
        Set that preserves ordering.

        From http://code.activestate.com/recipes/576694/
    """

    def __init__(self, iterable=None):
        self.end = end = []
        end += [None, end, end]         # sentinel node for doubly linked list
        self.map = {}                   # key --> [key, prev, next]
        if iterable is not None:
            self |= iterable

    def __len__(self):
        return len(self.map)

    def __contains__(self, key):
        return key in self.map

    def add(self, key):
        if key not in self.map:
            end = self.end
            curr = end[PREV]
            curr[NEXT] = end[PREV] = self.map[key] = [key, curr, end]

    def discard(self, key):
        if key in self.map:
            key, prev, next = self.map.pop(key)
            prev[NEXT] = next
            next[PREV] = prev

    def __iter__(self):
        end = self.end
        curr = end[NEXT]
        while curr is not end:
            yield curr[KEY]
            curr = curr[NEXT]

    def __reversed__(self):
        end = self.end
        curr = end[PREV]
        while curr is not end:
            yield curr[KEY]
            curr = curr[PREV]

    def pop(self, last=True):
        if not self:
            raise KeyError('set is empty')
        key = next(reversed(self)) if last else next(iter(self))
        self.discard(key)
        return key

    def __repr__(self):
        if not self:
            return '%s()' % (self.__class__.__name__,)
        return '%s(%r)' % (self.__class__.__name__, list(self))

    def __eq__(self, other):
        if isinstance(other, sset):
            return len(self) == len(other) and list(self) == list(other)
        return set(self) == set(other)

    def __del__(self):
        self.clear()   # remove circular references





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu



from functools import wraps


__all__ = ['import_from_path', 'attr', 'dynamicmethod', 'NullObject', 'Null']


def import_from_path(path):
    """
        Import a class dynamically, given it's dotted path.
    """
    module_name, class_name = path.rsplit('.', 1)
    try:
        return getattr(__import__(module_name, fromlist=[class_name]), class_name)
    except AttributeError:
        raise ImportError('Unable to import %s' % path)



def attr(obj, *attrs, **kwargs):
    """
        Follow chained attributes and get the value of the last attributes.
        If an attribute error is raised, returns the default value.

        res = attr(data, 'test', 'o', 'bla', default="yeah")

        is the equivalent of

        try:
            res = getattr(getattr(getattr(data, 'test'), 'o'), 'bla')
        except AttributeError:
            res = "yeah"

    """
    try:
        value = getattr(obj, attrs[0])

        for attr in attrs[1:]:
            value = getattr(value, attr)
    except (IndexError, AttributeError):
        return kwargs.get('default', None)

    return value


class InstanceAndClassMethodDescriptor(object):
    """
        Return a wrapper around the method calling the method as a classmethod
        or as a normal method depending of if it's called from an instance
        or a class.
    """

    def __init__(self, method):
        self.method = method

    def __get__(self, obj, obj_type):

        @wraps(self.method)
        def wrapper(*args, **kwargs):
            return self.method(obj or obj_type, *args, **kwargs)

        return wrapper



def dynamicmethod(func):
    """
        Wrap the method so it becomes a function wich will receives the first
        parameter as been the class or the instance depanding of if it's called
        from a class or an instance.
    """
    return InstanceAndClassMethodDescriptor(func)


class Singleton(type):
    """
        Metaclass implementing the Singleton pattern.

        Usage:

            >>> class Test(object):
            ...     __metaclass__ = Singleton
            ...
            >>> id(Test()) == id(Test())
            True
    """

    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]



class NullObject(object):
    """

        Null object pattern implementation. This object is a singleton. It
        accept any parameters, and attribute lookup and any method calls,
        and always return itself.

        Example:

            >>> n = NullObject()
            >>> n
            NullObject()
            >>> n == NullObject('value') == NullObject('value', param='value')
            True
            >>> n() == n('value') == n('value', param='value') == n
            True
            >>> n.attr1
            NullObject()
            >>> n.attr1.attr2
            NullObject()
            >>> n.method1()
            NullObject()
            >>> n.method1().method2()
            NullObject()
            >>> n.method('value')
            NullObject()
            >>> n.method(param='value')
            NullObject()
            >>> n.method('value', param='value')
            NullObject()
            >>> n.attr1.method1()
            NullObject()
            >>> n.method1().attr1
            NullObject()
            >>> n.attr1 = 'value'
            >>> n.attr1.attr2 = 'value'
            >>> del n.attr1
            >>> del n.attr1.attr2.attr3
            >>> str(n) == ''
            True
            >>> n + 1 / 7 % 3
            NullObject()
            >>> n[1] == n[:4] == n
            True
            >>> 'test' in n
            False
            >>> n['test']
            NullObject()
            >>> NullObject() >> 1
            NullObject()
            >>> NullObject() == None
            True


        Iterating on NullObject() will end up in a infinite loop with the next item
        always beeing NullObject(). Stuff like sorted() will hang.
    """

    def __init__(self, *args, **kwargs):
        """
            NullObject accept any arguments
        """
        pass

    def __repr__(self):
        return "NullObject()"

    def __str__(self):
        return ""

    def __eq__(self, other):
        """
            NullObject is only equal to itself or None
        """
        return isinstance(other, NullObject) or other is None

    # Like None, NullObject is False is a boolean context
    __nonzero__ = __bool__ = lambda self: False

    # Any attribute lookup, method call or operation on NullObject returns NullObject
    nullify = lambda self, *x, **kwargs: self

    __call__ = nullify
    __getattr__ = __setattr__ = __delattr__ = nullify
    __cmp__ = __ne__ = __lt__ = __gt__ = __le__ = __ge__ = nullify
    __pos__ = __neg__ = __abs__ = __invert__ = nullify
    __add__ = __sub__ = __mul__ = __mod__ = __pow__ = nullify
    __floordiv__ = __div__ = __truediv__ = __divmod__ = nullify
    __lshift__ = __rshift__ = __and__ = __or__ = __xor__ = nullify
    __radd__ = __rsub__ = __rmul__ = __rmod__ = __rpow__ = nullify
    __rfloordiv__ = __rdiv__ = __rtruediv__ = __rdivmod__ = nullify
    __rlshift__ = __rrshift__ = __rand__ = __ror__ = __rxor__ = nullify
    __iadd__ = __isub__ = __imul__ = __imod__ = __ipow__ = nullify
    __ifloordiv__ = __idiv__ = __itruediv__ = __idivmod__ = nullify
    __ilshift__ = __irshift__ = __iand__ = __ior__ = __ixor__ = nullify
    __getitem__ = __setitem__ = __delitem__ = nullify
    __getslice__ = __setslice__ = __delslice__ = nullify
    __reversed__ = nullify
    __contains__ = __missing__ = nullify
    __enter__ = __exit__ = nullify

    # Some spacial methods cannot be transformed because they should
    # return special types:
    # __int__ = __long__ = __float__ = __complex__ = __oct__ = __hex__ = ...
    # __index__ = __trunc__ = __coerce_ = ...
    # __len__ = ...
    # __iter__ = ...
    #  __round__ = __floor__ = __ceil__ = __trunc__ = ...


# One official instance off NullObject, that can be used like None
Null = type('Null', (NullObject,), {"__repr__": lambda s: "Null"})()
